@@ -1,14 +1,14 @@
-use super::display::print_cpu;
-use super::exit;
-use super::Error;
 use std::fs::File;
 use std::io::{Read, Write};
 use std::path::Path;
 
+use super::display::{print_cpu, render_cpu};
+use super::Error;
+
 pub trait Speed {
-    fn read_int(&mut self, sub_path: String) -> Result<i32, Error>;
-    fn read_str(&mut self, sub_path: String) -> Result<String, Error>;
-    fn read_temp(&mut self, sub_path: String) -> Result<i32, Error>;
+    fn read_int(&mut self, sub_path: &str) -> Result<i32, Error>;
+    fn read_str(&mut self, sub_path: &str) -> Result<String, Error>;
+    fn read_temp(&mut self, sub_path: &str) -> Result<i32, Error>;
     fn write_value(&mut self, value: WritableValue) -> Result<(), Error>;
     fn update(&mut self) -> Result<(), Error>;
     fn init_cpu(&mut self) -> Result<(), Error>;
@@ -21,11 +21,13 @@ pub trait Speed {
     fn get_gov(&mut self) -> Result<(), Error>;
     fn set_gov(&mut self, gov: String) -> Result<(), Error>;
     fn print(&self);
+    fn render(&self) -> String;
 }
 
 #[derive(Debug)]
 pub struct CPU {
     pub name: String,
+    pub number: i8,
     pub max_freq: i32,
     pub min_freq: i32,
     pub cur_freq: i32,
@@ -42,30 +44,38 @@ pub enum WritableValue {
 
 impl Speed for CPU {
     /// A generic function to take a path and a single cpu (single core) and get an i32
-    fn read_int(&mut self, sub_path: String) -> Result<i32, Error> {
+    fn read_int(&mut self, sub_path: &str) -> Result<i32, Error> {
         let mut info: String = String::new();
         let cpu_info_path: String = format!("/sys/devices/system/cpu/{}/{}", self.name, sub_path);
 
         File::open(cpu_info_path)?.read_to_string(&mut info)?;
 
-        // Remove the last character (the newline)
+        // Remove newline
         info.pop();
-        match info.parse::<i32>() {
-            Err(_) => {
-                eprintln!("Could not read {}", sub_path);
-                exit(1);
-            }
-            Ok(a) => Ok(a),
-        }
+        Ok(info
+            .parse::<i32>()
+            .unwrap_or_else(|e| panic!("Could not parse {}\n{}", sub_path, e)))
     }
 
-    fn read_temp(&mut self, sub_path: String) -> Result<i32, Error> {
+    fn read_str(&mut self, sub_path: &str) -> Result<String, Error> {
+        let mut info: String = String::new();
+        let cpu_info_path: String = format!("/sys/devices/system/cpu/{}/{}", self.name, sub_path);
+
+        File::open(cpu_info_path)?.read_to_string(&mut info)?;
+
+        // Remove newline
+        info.pop();
+        Ok(info)
+    }
+
+    fn read_temp(&mut self, sub_path: &str) -> Result<i32, Error> {
         let mut info: String = String::new();
         let cpu_info_path: String = format!(
             "/sys/class/thermal/{}/{}",
             self.name.replace("cpu", "thermal_zone"),
             sub_path
         );
+
         if !Path::new(&cpu_info_path).exists() {
             return Ok(-1);
         }
@@ -75,24 +85,9 @@ impl Speed for CPU {
         // Remove the last character (the newline)
         info.pop();
 
-        match info.parse::<i32>() {
-            Err(_) => {
-                eprintln!("Could not read {}", sub_path);
-                exit(1);
-            }
-            Ok(a) => Ok(a),
-        }
-    }
-
-    fn read_str(&mut self, sub_path: String) -> Result<String, Error> {
-        let mut info: String = String::new();
-        let cpu_info_path: String = format!("/sys/devices/system/cpu/{}/{}", self.name, sub_path);
-
-        File::open(cpu_info_path)?.read_to_string(&mut info)?;
-
-        // Remove the last character (the newline)
-        info.pop();
-        Ok(info)
+        Ok(info
+            .parse::<i32>()
+            .unwrap_or_else(|e| panic!("Could not parse {}\n{}", sub_path, e)))
     }
 
     fn write_value(&mut self, value: WritableValue) -> Result<(), Error> {
@@ -150,32 +145,27 @@ impl Speed for CPU {
     }
 
     fn get_max(&mut self) -> Result<(), Error> {
-        let path = "cpufreq/scaling_max_freq";
-        self.max_freq = self.read_int(path.to_string())?;
+        self.max_freq = self.read_int("cpufreq/scaling_max_freq")?;
         Ok(())
     }
 
     fn get_min(&mut self) -> Result<(), Error> {
-        let path = "cpufreq/scaling_min_freq";
-        self.min_freq = self.read_int(path.to_string())?;
+        self.min_freq = self.read_int("cpufreq/scaling_min_freq")?;
         Ok(())
     }
 
     fn get_cur(&mut self) -> Result<(), Error> {
-        let path = "cpufreq/scaling_cur_freq";
-        self.cur_freq = self.read_int(path.to_string())?;
+        self.cur_freq = self.read_int("cpufreq/scaling_cur_freq")?;
         Ok(())
     }
 
     fn get_temp(&mut self) -> Result<(), Error> {
-        let path = "temp";
-        self.cur_temp = self.read_temp(path.to_string())?;
+        self.cur_temp = self.read_temp("temp")?;
         Ok(())
     }
 
     fn get_gov(&mut self) -> Result<(), Error> {
-        let path = "cpufreq/scaling_governor";
-        self.gov = self.read_str(path.to_string())?;
+        self.gov = self.read_str("cpufreq/scaling_governor")?;
         Ok(())
     }
 
@@ -188,4 +178,8 @@ impl Speed for CPU {
     fn print(&self) {
         print_cpu(self);
     }
+
+    fn render(&self) -> String {
+        render_cpu(self)
+    }
 }
