@@ -17,17 +17,16 @@ impl From<std::io::Error> for Error {
 
 impl fmt::Debug for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(
-            f,
-            "Something went wrong. Try running with sudo.\n"
-        )
+        write!(f, "Something went wrong. Try running with sudo.\n")
     }
 }
 
 pub struct GovSetError;
+
 pub struct SpeedSetError;
 
 pub struct GovGetError;
+
 pub struct SpeedGetError;
 
 pub struct TempGetError;
